"use strict";Object.defineProperty(exports, "__esModule", {value: true});// src/index.ts
var _unplugin = require('unplugin');
var _pluginutils = require('@rollup/pluginutils');





var _common = require('@vue-macros/common');
var _reactivitytransform = require('@vue/reactivity-transform');

// src/core/index.ts











// src/core/helper.ts
var helperId = "/vue2-reactivity-transform-helper";
var helperCode = `export function createPropsRestProxy(props, excludedKeys) {
  const ret = {}
  for (const key in props) {
    if (!excludedKeys.includes(key)) {
      Object.defineProperty(ret, key, {
        enumerable: true,
        get: () => props[key],
      })
    }
  }
  return ret
}`;

// src/core/index.ts
function transfromVueSFC(code, id) {
  const s = new (0, _common.MagicString)(code);
  const { script, scriptSetup } = _common.parseSFC.call(void 0, code, id);
  let refBindings;
  let propsDestructuredBindings;
  if (script && _reactivitytransform.shouldTransform.call(void 0, script.content)) {
    const ast = _common.babelParse.call(void 0, script.content, script.lang);
    const offset = script.loc.start.offset;
    const { importedHelpers, rootRefs } = _reactivitytransform.transformAST.call(void 0, ast, s, offset);
    refBindings = rootRefs;
    addHelpers(s, script.loc.start.offset, importedHelpers);
  }
  if (scriptSetup) {
    const ast = _common.babelParse.call(void 0, scriptSetup.content, scriptSetup.lang);
    for (const node of ast.body) {
      processDefineProps(node);
    }
    if (propsDestructuredBindings || refBindings || _reactivitytransform.shouldTransform.call(void 0, scriptSetup.content)) {
      const { importedHelpers } = _reactivitytransform.transformAST.call(void 0, 
        ast,
        s,
        scriptSetup.loc.start.offset,
        refBindings,
        propsDestructuredBindings
      );
      addHelpers(s, scriptSetup.loc.start.offset, importedHelpers);
    }
  }
  return _common.getTransformResult.call(void 0, s, id);
  function processDefineProps(node) {
    if (node.type !== "VariableDeclaration")
      return;
    const decl = node.declarations.find(
      (decl2) => _common.isCallOf.call(void 0, decl2.init, _common.DEFINE_PROPS)
    );
    if (!decl || decl.id.type !== "ObjectPattern")
      return;
    if (node.declarations.length > 1)
      throw new SyntaxError(
        `${_common.DEFINE_PROPS}() don't support multiple declarations.`
      );
    const offset = scriptSetup.loc.start.offset;
    let defaultStr = "";
    propsDestructuredBindings = {};
    for (const prop of decl.id.properties) {
      if (prop.type === "ObjectProperty") {
        const propKey = _common.resolveObjectKey.call(void 0, prop.key, prop.computed, false);
        if (!propKey) {
          throw new SyntaxError(
            `${_common.DEFINE_PROPS}() destructure cannot use computed key.`
          );
        }
        if (prop.value.type === "AssignmentPattern") {
          const { left, right } = prop.value;
          if (left.type !== "Identifier") {
            throw new SyntaxError(
              `${_common.DEFINE_PROPS}() destructure does not support nested patterns.`
            );
          }
          propsDestructuredBindings[propKey] = {
            local: left.name
          };
          defaultStr += `${propKey}: ${s.sliceNode(right, { offset })},`;
        } else if (prop.value.type === "Identifier") {
          propsDestructuredBindings[propKey] = {
            local: prop.value.name
          };
        } else {
          throw new SyntaxError(
            `${_common.DEFINE_PROPS}() destructure does not support nested patterns.`
          );
        }
      } else {
        s.prependLeft(
          offset,
          `import { createPropsRestProxy } from '${helperId}';
const ${prop.argument.name} = createPropsRestProxy(__props, ${JSON.stringify(
            Object.keys(propsDestructuredBindings)
          )});
`
        );
      }
    }
    const declStr = `withDefaults(${s.sliceNode(decl.init, {
      offset
    })}, { ${defaultStr} })`;
    s.overwriteNode(node, declStr, { offset });
  }
}
function addHelpers(s, offset, helpers) {
  if (helpers.length === 0)
    return;
  s.prependLeft(
    offset,
    `import { ${helpers.map((h) => `${h} as _${h}`).join(", ")} } from 'vue';
`
  );
}

// src/index.ts
function resolveOption(options, framework) {
  return {
    include: [_common.REGEX_SRC_FILE, _common.REGEX_VUE_SFC, _common.REGEX_SETUP_SFC].concat(
      framework === "webpack" ? _common.REGEX_VUE_SUB : []
    ),
    exclude: [/node_modules/],
    ...options
  };
}
var name = "unplugin-reactivity-transform-vue2";
var src_default = _unplugin.createUnplugin.call(void 0, 
  (userOptions = {}, { framework }) => {
    const options = resolveOption(userOptions, framework);
    const filter = _pluginutils.createFilter.call(void 0, options.include, options.exclude);
    return {
      name,
      enforce: "pre",
      resolveId(id) {
        if (id === helperId)
          return id;
      },
      loadInclude(id) {
        return id === helperId;
      },
      load(_id) {
        const id = _pluginutils.normalizePath.call(void 0, _id);
        if (id === helperId)
          return helperCode;
      },
      transformInclude(id) {
        return filter(id);
      },
      transform(code, id) {
        try {
          if (_common.REGEX_VUE_SFC.test(id) || _common.REGEX_SETUP_SFC.test(id) || framework === "webpack" && _common.REGEX_VUE_SUB.test(id)) {
            return transfromVueSFC(code, id);
          } else {
            if (!_reactivitytransform.shouldTransform.call(void 0, code))
              return;
            return _reactivitytransform.transform.call(void 0, code, {
              filename: id,
              sourceMap: true
            });
          }
        } catch (err) {
          this.error(`${name} ${err}`);
        }
      }
    };
  }
);



exports.src_default = src_default;
